/***
 *
 * Copyright (C) 2003-2004 Fredrik Lingvall (fl@signal.uu.se)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ***/

/***
 *
 * Fredrik Lingvall 2004-01-21 : First release.
 *
 *
 ***/

#include "config.h"

#include "i18n.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <xmms/util.h>
#include <xmms/configfile.h>
#include "filter.h"

static const char *filter_about_text =
N_("FIR Filter Plugin\n"
   "Version " VERSION
   "\nBy Fredrik Lingvall <fl@signal.uu.se>.");

static GtkWidget *conf_dialog = NULL;
static GtkObject *filter_gain_adj;
static GtkEntry  *filter_file_name;
static gchar     browser_fname[256];

/***
 *
 * About dialog
 *
 ***/

void filter_about(void)
{ 
  static GtkWidget *filter_about_dialog = NULL;
  
  if (filter_about_dialog != NULL)
    return;
  
  filter_about_dialog = xmms_show_message(_("About FIR Filter Plugin"),
					  _(filter_about_text), _("Ok"),
					  FALSE, NULL, NULL);
  gtk_signal_connect(GTK_OBJECT(filter_about_dialog), "destroy",
		     GTK_SIGNAL_FUNC(gtk_widget_destroyed),
		     &filter_about_dialog);
} 

/***
 *
 * Error dialog
 *
 ***/

void filter_error(gchar *err_txt)
{
  static GtkWidget *filter_err_dialog = NULL;
  
  if (filter_err_dialog != NULL)
    return;
  
  filter_err_dialog = xmms_show_message(_(""),
					_(err_txt), _("Close"),
					FALSE, NULL, NULL);
  gtk_signal_connect(GTK_OBJECT(filter_err_dialog), "destroy",
		     GTK_SIGNAL_FUNC(gtk_widget_destroyed),
		     &filter_err_dialog);
}

/***
 *
 *  File browser stuff
 *
 ***/

static void file_ok_sel(GtkWidget *ok_button, GtkWidget *w)
{
  static gchar *tmpStr;

  tmpStr = gtk_file_selection_get_filename(GTK_FILE_SELECTION(w));
  strcpy(browser_fname,tmpStr);

  gtk_widget_destroy(GTK_WIDGET(w));

  // Make the configure dialog responsive to user actions again.
  gtk_widget_set_sensitive(conf_dialog,TRUE);

  // Set the file name text in the configure dialog.
  gtk_entry_set_text(filter_file_name,browser_fname);
}

static void file_cancel_sel(GtkWidget *sel_button, GtkWidget *w)
{
  // Just use the old file.
  strcpy(filter_file, browser_fname);

  gtk_widget_destroy(GTK_WIDGET(w));

  // Make the configure dialog responsive to user actions again.
  gtk_widget_set_sensitive(conf_dialog,TRUE);
  //gtk_widget_set_style(conf_dialog,GTK_STATE_ACTIVE);
}

static void filter_filebrowser(void)
{
  static GtkWidget *filew = NULL;
  
  // If the configure dialog isn't running bail out.
  if (conf_dialog == NULL)
    return;
  
  // Make the configure dialog unresponsive to user actions.
  gtk_widget_set_sensitive(conf_dialog,FALSE);

  // Init the browser_fname string.
  strcpy(browser_fname,"");

  // Create a new file selection widget.
  filew = gtk_file_selection_new("FIR filter file selection");
  
  // Connect the ok_button to file_ok_sel function.
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),
  	    "clicked", GTK_SIGNAL_FUNC(file_ok_sel), (gpointer) filew);
  
  // Connect the cancel_button to destroy the widget.
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button),
  		     "clicked", GTK_SIGNAL_FUNC(file_cancel_sel), filew);

  printf("%s\n", filter_file);

  // Set a default filename in the edit text box.
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), filter_file);
  
  gtk_widget_show (filew);
}

/***
 *
 *  Configure stuff.
 *
 ***/

static void apply_changes(void)
{
  ConfigFile *cfg;
  
  filter_gain = GTK_ADJUSTMENT(filter_gain_adj)->value;
  filter_file = gtk_entry_get_text(filter_file_name);

  // Open the ~/.xmms/config file.  
  cfg = xmms_cfg_open_default_file();

  xmms_cfg_write_int(cfg, "filter_plugin", "filter_gain", filter_gain);
  xmms_cfg_write_string(cfg, "filter_plugin", "filter_file", filter_file);

  // Write to  the ~/.xmms/config file.  
  xmms_cfg_write_default_file(cfg);

  // Close the file.
  xmms_cfg_free(cfg);
}

static void conf_ok_cb(GtkButton * button, gpointer data)
{
  apply_changes();
  gtk_widget_destroy(GTK_WIDGET(conf_dialog));
}

static void conf_reload_cb(GtkButton * button, gpointer data)
{
  FILE       *tmpFile;

  filter_file = gtk_entry_get_text(filter_file_name);

  // Check if the file exists.
  if (!(tmpFile = fopen(filter_file, "r" )))
    filter_error("FIR Plugin error:  Unable to open FIR coefficents file.");
  else
    fclose(tmpFile);
  
  // Set the reload flag.
  reload_filter = TRUE;
}

static void conf_cancel_cb(GtkButton * button, gpointer data)
{
  gtk_widget_destroy(GTK_WIDGET(conf_dialog));
}

static void conf_apply_cb(GtkButton * button, gpointer data)
{
  apply_changes();
}

static void conf_browse_cb(GtkButton *button, gpointer data)
{
  filter_filebrowser();

  // Change the filename if its different from old name.
  if (strcmp(browser_fname,filter_file) != 0)
    gtk_entry_set_text(filter_file_name,browser_fname);
}

//
// The configure dialog.
//

void filter_configure(void)
{
  GtkWidget *button, *table, *label, *hscale, *bbox;
  ConfigFile *cfg;
  gint       n;

  if (conf_dialog != NULL)
    return;

  //
  // Read the the XMMS config file.
  //

  if(!filter_file)
    filter_file = (char*) g_malloc0(256*sizeof(char));
  
  if (sizeof(short) != sizeof(gint16))
    abort();
  
  // Open the ~/.xmms/config file.
  cfg = xmms_cfg_open_default_file();
  
  // Read plugin paremeters.
  xmms_cfg_read_boolean(cfg, "filter_plugin", "reload_filter", &reload_filter);
  xmms_cfg_read_string(cfg, "filter_plugin", "filter_file", &filter_file);
  
  // Close the config file.
  xmms_cfg_free(cfg);

  //
  // Create the configure dialog.
  //

  conf_dialog = gtk_dialog_new();
  gtk_signal_connect(GTK_OBJECT(conf_dialog), "destroy",
		     GTK_SIGNAL_FUNC(gtk_widget_destroyed), &conf_dialog);
  gtk_window_set_title(GTK_WINDOW(conf_dialog), _("Configure FIR Filter"));

  filter_gain_adj = gtk_adjustment_new(filter_gain, 1, 100, 2, 10, 1);

  filter_file_name = gtk_entry_new();


  // Table.
  table = gtk_table_new(2, 3, FALSE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(conf_dialog)->vbox), table,
  	     TRUE, TRUE, 5);
  gtk_widget_show(table);


  // Filter gain text.
  label = gtk_label_new(_("Filter Gain (x 10) :"));
  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
  gtk_widget_show(label);

  // Filter gain slider.
  hscale = gtk_hscale_new(GTK_ADJUSTMENT(filter_gain_adj));
  gtk_widget_set_usize(hscale, 400, 35);
  gtk_scale_set_digits(GTK_SCALE(hscale), 0);
  gtk_table_attach_defaults(GTK_TABLE(table), hscale, 1, 2, 0, 1);
  gtk_widget_show(hscale);


  //
  // Check box.
  //

  /*
  surround_btn = gtk_check_button_new_with_label(_("Surround filter"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(surround_btn),
  filter_surround_enable);
  gtk_widget_show(surround_btn);
  
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(conf_dialog)->vbox), surround_btn,
  TRUE, TRUE, 5);
  */


  // 
  // Editable text box.
  //	

  label = gtk_label_new(_("FIR coefficents file:"));
  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
  gtk_widget_show(label);

  gtk_entry_set_text(filter_file_name,filter_file);
  gtk_table_attach_defaults(GTK_TABLE(table), filter_file_name, 1, 2, 1, 2);
  gtk_widget_show(filter_file_name);
                         
  //
  // Buttons.
  //

  // ... button (file browser).
  button = gtk_button_new_with_label(_("..."));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(conf_browse_cb),filter_file_name);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 2, 3, 1, 2); // Put in the Table.
  gtk_widget_show(button);
  

  // Start button box.
  bbox = gtk_hbutton_box_new();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
  gtk_box_pack_start(GTK_BOX((GTK_DIALOG(conf_dialog)->action_area)),
		     bbox, TRUE, TRUE, 0);

  // OK button.
  button = gtk_button_new_with_label(_("Ok"));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(bbox), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(conf_ok_cb), NULL);
  gtk_widget_grab_default(button);
  gtk_widget_show(button);
  
  // Reload filter button.
  button = gtk_button_new_with_label(_("Reload FIR filter"));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(bbox), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(conf_reload_cb), NULL);
  gtk_widget_show(button);

  // Cancel button.
  button = gtk_button_new_with_label(_("Cancel"));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(bbox), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(conf_cancel_cb), NULL);
  gtk_widget_show(button);
  
  // Apply button.
  button = gtk_button_new_with_label(_("Apply"));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(bbox), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(conf_apply_cb), NULL);
  gtk_widget_show(button);


  // Show the button box.
  gtk_widget_show(bbox);

  gtk_widget_show(conf_dialog);
}
